fogbugz_tree = {}

local function debug(A, B, C, D) end

function fogbugz_tree.findArticlePath(ArticleMap, ArticleId)
   local PathList = {}
   local PathMap = {} -- we record all we found so far into a map, to detect duplicate and avoid risk of looping
   local CurrentId = ArticleId
   while CurrentId and not PathMap[CurrentId] do
      table.insert(PathList, 1, CurrentId)
      PathMap[CurrentId] = true
      local CurrentArticle = ArticleMap[CurrentId]
      debug(CurrentArticle)
      if CurrentArticle and CurrentArticle.ParentId then
         CurrentId = CurrentArticle.ParentId
      else
         break
      end
   end
   return PathList
end

local function tableIndex(Table, SearchValue, DefaultIndex)
   for Index,Value in ipairs(Table) do
      if Value == SearchValue then
         return Index
      end
   end
   return DefaultIndex or -1
end

function fogbugz_tree.createArticleMap(ArticlesNode)
   local ArticleIndex
   local ArticleMap = {}
   for ArticleIndex=1,ArticlesNode:childCount("article") do
      local ArticleNode = ArticlesNode:child("article", ArticleIndex)
      local Article = {
         Id=ArticleNode.ixWikiPage[1]:nodeValue(),
         Title=ArticleNode.sHeadline[1]:nodeValue(),
         ChildList={}
      }
      debug(Article)
      ArticleMap[Article.Id] = Article
   end
   return ArticleMap
end

function fogbugz_tree.connectLinks(ArticleMap)
   local ArticleId, Article
   for ArticleId, Article in pairs(ArticleMap) do
      Article.ParentId = fogbugz_parse.findParent(Article.RawHtml)
      Article.LinkList = fogbugz_parse.findLinks(Article.RawHtml)
      if ArticleMap[Article.ParentId] then
         table.insert(ArticleMap[Article.ParentId].ChildList, Article.Id)
      end
      if ArticleId == '4111' or ArticleId=='4109' then debug(ArticleId, Article) end
   end
end

function fogbugz_tree.computePathAndSortChildList(ArticleMap)
   local ArticleId, Article
   for ArticleId, Article in pairs(ArticleMap) do
      Article.PathList = fogbugz_tree.findArticlePath(ArticleMap, Article.Id)
      local LinkList = Article.LinkList
      local OrderFunc = function(Key1, Key2) return tableIndex(LinkList,Key1, #LinkList+1) < tableIndex(LinkList,Key2, #LinkList+1) end
      table.sort(Article.ChildList, OrderFunc)
      debug(Article)
   end
end

function fogbugz_tree.rootIdList(ArticleMap, StartPageId)
   local RootIdList = {}
   for ArticleId, Article in pairs(ArticleMap) do
      if not Article.ParentId then
         table.insert(RootIdList, ArticleId)
      end
   end
   if StartPageId and ArticleMap[StartPageId] then
      -- top-level pages are a bit of a special case -
      -- the way we parse out ParentId from the CookieCrumbs, it doesn't return StartPageId for 1st level pages,
      -- although it's really set to that
      -- as a result, 1st level pages appear "orphaned"
      -- this work fine for the purpose of having the 1st level pages appear at the top level, instead of 1 level deeper,
      -- but we need to get them sorted - by the order in which they appear in StartPage
      local RootOrderList = { StartPageId } -- we stick in StartPageId itself in front, so that it appears first
      local LinkIndex, LinkId
      for LinkIndex, LinkId in ipairs(ArticleMap[StartPageId].LinkList or {}) do
         table.insert(RootOrderList, LinkId)
      end
      debug(RootOrderList)
      local LinkList = RootOrderList
      local OrderFunc = function(Key1, Key2) return tableIndex(LinkList,Key1, #LinkList+1) < tableIndex(LinkList,Key2, #LinkList+1) end
      table.sort(RootIdList, OrderFunc)
   else
      table.sort(RootIdList, function(RootId1, RootId2) return (0+RootId1) < (0+RootId2) end)
   end
   return RootIdList
end

local function articleToOrderList(Article, ArticleMap, OrderList)
   table.insert(OrderList, Article.Id)
   local ChildIndex, ChildId
   for ChildIndex, ChildId in ipairs(Article.ChildList) do
      local ChildArticle = ArticleMap[ChildId]
      articleToOrderList(ChildArticle, ArticleMap, OrderList)
   end
end

function fogbugz_tree.articleMapToOrderList(ArticleMap, StartPageId)
   local RootIdList = fogbugz_tree.rootIdList(ArticleMap, StartPageId)
   local OrderList = {}
   local RootIndex, RootId
   for RootIndex, RootId in ipairs(RootIdList) do
      articleToOrderList(ArticleMap[RootId], ArticleMap, OrderList)
   end
   return OrderList
end

function fogbugz_tree.setPrevNext(ArticleMap, OrderList)
   local OrderIndex, ArticleId
   for OrderIndex, ArticleId in ipairs(OrderList) do
      local Article = ArticleMap[ArticleId]
      if OrderIndex > 1 then
         Article.PrevId = OrderList[OrderIndex-1]
      end
      if OrderIndex < #OrderList then
         Article.NextId = OrderList[OrderIndex+1]
      end
      debug(Article)
   end
end

function fogbugz_tree.prepareArticleMap(ArticleMap, StartPageId)
   fogbugz_tree.connectLinks(ArticleMap)
   fogbugz_tree.computePathAndSortChildList(ArticleMap)
   local OrderList = fogbugz_tree.articleMapToOrderList(ArticleMap, StartPageId)
   fogbugz_tree.setPrevNext(ArticleMap, OrderList)
end
