require('fogbugz_parse')

fogcache_gen = {}

local function debug(A, B, C, D) end

local DefaultSuffix = '.new'
fogcache_gen.DefaultSuffix = DefaultSuffix

function fogcache_gen.cookieCrumbLinks(Html, Config) 

   local StartPage = Config.START_PAGE or '1'
   local Suffix = Config.SUFFIX or DefaultSuffix
   local BreadCrumbsHtmlSanitized = fogbugz_parse.sanitizeHtmlForXml( fogbugz_parse.findBreadCrumbsTable(Html) )
   debug(BreadCrumbsHtmlSanitized)

   local Xml = xml.parse{data=BreadCrumbsHtmlSanitized}
   debug(Xml)

   local Output = '<a href="'..StartPage..Suffix..'.html">'..Xml.table.tr.td.a.title:nodeValue()..'</a>'

   local LinkCount = Xml.table.tr:child("td",2):childCount('a')
   
   for LinkIndex = 1, LinkCount do
      local LinkNode = Xml.table.tr:child("td", 2):child("a", LinkIndex)
      local Href = LinkNode.href:nodeValue():gsub('default%.asp%?W(%d*)', '%1'..Suffix..'%.html')
      local Text = LinkNode.title:nodeValue()
      Output = Output..' &rsaquo; '
      if i ~= LinkCount then
         Output = Output..'<a href="'..Href..'">'..Text..'</a>'
      else
         Output = Output..Text
      end
   end
   return Output
end

function fogcache_gen.staticTreeview(ArticlePath, TreeNodeList, PathDepth, Suffix)
   local Result = ''
   if not PathDepth then PathDepth = 1 end
   for TreeNodeIndex=1, #TreeNodeList do
      local TreeNode = TreeNodeList[TreeNodeIndex]
      local ThisArticleId = TreeNode.id:sub(2)
      local ThisArticleTitle = TreeNode.title
      local ArticleNode = xml.parse{data=
         '<li class="leaf">\n<div>\n<span class="treeIcon fake">\n</span>\n<span id="bnw_'..ThisArticleId..
         '" class="nodeText bnw_article">\n<a href="'..ThisArticleId..
         Suffix..'.html" title=""></a>\n</span>\n</div>\n<ul style="display:none;"></ul>\n</li>'}[1]
      
      -- Setting attributes/text like this will do proper XML/HTML escaping.
      local ATag = ArticleNode.div:child("span", 2).a
      ATag.title = ThisArticleTitle
      ATag:append(xml.TEXT, ThisArticleTitle)
      
      if #TreeNode.children > 0 then
         if ArticlePath and ArticlePath[PathDepth] == ThisArticleId then
            ArticleNode.class = 'collapsable'
            ArticleNode.ul.style = ''
            local ChildrenHtml = fogcache_gen.staticTreeview(ArticlePath, TreeNode.children, PathDepth+1, Suffix)
            debug('ChildrenHtml: ', ChildrenHtml)
            ChildrenHtml = ChildrenHtml..'<li style="display:none"></li>' -- workaround for #18358
            ArticleNode.ul:setInner(ChildrenHtml)
         else
            ArticleNode.class = 'expandable'
         end
      end
      if ArticlePath and ThisArticleId == ArticlePath[#ArticlePath] then
         local NodeTextSpan = ArticleNode.div:child('span', 2)
         NodeTextSpan.class = NodeTextSpan.class..' c'
      end
      Result = Result..tostring(ArticleNode)..'\n'
   end
   return Result
end

local function parseAttachmentLinkSrc(ImageLinkSrc, Config)
   local AttachmentId = ImageLinkSrc:gsub('.*ixAttachment=([0-9]+)&.*', '%1')
   local AttachmentFileName = ImageLinkSrc:gsub('.*;sFileName=([^;"]*)"', '%1')
   local FullLink = Config.BASE_URL..'/default.asp?' .. ImageLinkSrc:gsub('[a-z]*=\"default.asp%?([^"]*)\"', '%1'):gsub('&amp;','&')
   return AttachmentId, AttachmentFileName, FullLink
end

local function saveAttachment(AttachmentId, AttachmentFileName, FullLink, Config, Token)
   -- determine dir and filename
   local AttachmentDir = Config.DEST_PATH .. '/files/' .. AttachmentId
   local AttachmentDirFileName = AttachmentDir .. '/' .. AttachmentFileName
   debug(AttachmentDir, AttachmentDirFileName)
   
   -- if we already have a non-empty file, that's good enough - skip it
   if file_helper.fileSize(AttachmentDirFileName) then
      return
   end
   
   -- fetch the body, measure time it takes
   local TimeStart = os.time()
   local AttachmentBody = fogbugz_api.readAttachment(FullLink, Token)
   local TimeEnd = os.time()
   local TimeDeltaSeconds = (TimeEnd-TimeStart)*3600*24
   debug(TimeDeltaSeconds)
   if not AttachmentBody then
      debug('Took '..TimeDeltaSeconds..'s for '..AttachmentFileName..' - failed')
      return -- ignoring the error - nothing we can do here, but the show must go on
   end
   debug('Took '..TimeDeltaSeconds..'s for '..AttachmentFileName..' - succeeded')
   
   -- save the body
   os.execute('mkdir -p ' .. AttachmentDir)
   file_helper.writeFile(AttachmentDirFileName, AttachmentBody)
end

function processAttachmentsFindFetchReplace(Html, Config, Token)
   local ProcessedHtml = Html
   local ImageLinkSrc
   
   for ImageLinkSrc in Html:gfind('src=\"default.asp[^\"]*"') do
      local AttachmentId, AttachmentFileName, FullLink = parseAttachmentLinkSrc(ImageLinkSrc, Config)
      local ReplacementImageLinkSrc = 'src="files/' .. AttachmentId .. '/' .. AttachmentFileName:gsub('%%', '%%25') .. '"'
      saveAttachment(AttachmentId, AttachmentFileName, FullLink, Config, Token)
      local PosStart, PosEnd = ProcessedHtml:find(ImageLinkSrc,1,true)
      ProcessedHtml = ProcessedHtml:sub(1, PosStart-1) .. ReplacementImageLinkSrc .. ProcessedHtml:sub(PosEnd+1)
   end
   
   for FileLinkSrc in Html:gfind('href=\"default.asp%?pg=pgDownload&amp;pgType=pgWikiAttachment&amp;[^\"]*"') do
      local AttachmentId, AttachmentFileName, FullLink = parseAttachmentLinkSrc(FileLinkSrc, Config)
      local ReplacementFileLinkSrc = 'href="files/' .. AttachmentId .. '/' .. AttachmentFileName:gsub('%%', '%%25') .. '"'
      saveAttachment(AttachmentId, AttachmentFileName, FullLink, Config, Token)
      local PosStart, PosEnd = ProcessedHtml:find(FileLinkSrc,1,true)
      ProcessedHtml = ProcessedHtml:sub(1, PosStart-1) .. ReplacementFileLinkSrc .. ProcessedHtml:sub(PosEnd+1)
   end
   
   return ProcessedHtml
end


function fogcache_gen.fullPage(Article, TreeData, TreeHash, Config, Token)
   
   local PageId=Article.Id
   local Html =  Article.RawHtml
   
   local Suffix = Config.SUFFIX or DefaultSuffix

   local CookieCrumbs = fogcache_gen.cookieCrumbLinks(Article.RawHtml, Config)
   local StaticTreeview = fogcache_gen.staticTreeview(Article.PathList, TreeData, nil, Suffix)

   local Title = Html:sub(Html:find('%<title%>.*%</title%>'))
   
   local ContentStart = Html:find('%<div id="startBody"%>%</div%>',1)
   local ContentFinish = Html:find('%<div id="endBody"%>%</div%>')
   local Content = Html:sub(ContentStart, ContentFinish-1)
   
   Content = Content:gsub('default%.asp%?W(%d*)', '%1%'..Suffix..'.html')
   Content = Content:gsub('href="default%.asp%?pg=pgWikiView&amp;ixWiki=1&amp;sView=recently_changed"', 'href="RecentlyChanged.1.html"')
   Content = Content:gsub('https:%/%/interfaceware%.fogbugz%.com%/', '')
   Content = processAttachmentsFindFetchReplace(Content, Config, Token)

   local NavHtml = ''
   local NavHtmlArrow = ''

   local LabelArrowPrev = Config.LABEL_ARROW_PREV or '&larr; Prev'
   local LabelArrowNext = Config.LABEL_ARROW_NEXT or 'Next &rarr;'
   local LabelArrowSep = Config.LABEL_ARROW_SEP or ' | '

   if Article.PrevId then
      NavHtml = NavHtml .. ' <a href="'..Article.PrevId..Suffix..'.html">Prev</a> '
      NavHtmlArrow = NavHtmlArrow .. '<a href="'..Article.PrevId..Suffix..'.html">'..LabelArrowPrev..'</a>'
   else 
      NavHtml = NavHtml .. ' Prev '
      NavHtmlArrow = NavHtmlArrow .. LabelArrowPrev
   end

   if Article.NextId then
      NavHtml = NavHtml .. ' | <a href="'..Article.NextId..Suffix..'.html">Next</a>'
      NavHtmlArrow = NavHtmlArrow .. LabelArrowSep .. '<a href="'..Article.NextId..Suffix..'.html">'..LabelArrowNext..'</a>'
   else
      NavHtml = NavHtml .. ' | Next'
      NavHtmlArrow = NavHtmlArrow .. LabelArrowSep .. LabelArrowNext
   end
   debug(NavHtml)
   
   PageData = {
      Id = PageId,
      Title = Title,
      TreeHash = TreeHash,
      ArticlePath = json.serialize{data=Article.PathList or {} },
      NavHtml = NavHtml,
      NavHtmlArrow = NavHtmlArrow,
      StartPage = Config.START_PAGE or '1',
      EditLink = Config.editLink(PageId),
      Suffix = Suffix,
      StaticTreeview = StaticTreeview,
      CookieCrumbs = CookieCrumbs,
      Content=Content,
   }
   
   return Config.generatePage(Config, PageData)
   
end
