require('fogbugz_api')
require('fogbugz_parse')
require('file_helper')
require('fogbugz_tree')
require('fogcache_files')
require('fogcache_tree')
require('fogcache_gen')
require('fogcache_tree')
require('fogcache_clean')
require('fogcache_sitemap')

local function debug(A, B, C, D) end

local Token
local UiSession

fogcache_main = {}

local function parseQueryString(QueryString)
   -- hacky!
   local Request = net.http.parseRequest{data='GET /?'..QueryString..' HTTP/1.1\r\n\r\n'}
   return Request.get_params
end

function fogcache_main.handleQueryString(Config, QueryString)
   if QueryString == 'noop' then
      return
   end
   local Query = parseQueryString(QueryString)
   debug(Query)
   return fogcache_main.construct(Config, Query)
end

function fogcache_main.construct(Config, Query)
   debug('Query:', Query)
   
   local FetchPageId = Query.FetchPageId
   debug('FechPageId:', FetchPageId)
   
   Token = fogbugz_api.logonIfNeeded(Config.BASE_URL, Config.EMAIL, Config.PASSWORD, Token)
   UiSession = fogbugz_api.uiLogonIfNeeded(Config.BASE_URL, Config.EMAIL, Config.PASSWORD, UiSession)
   
   local ArticlesXmlRoot = fogbugz_api.listArticles(Config.BASE_URL, Token, Config.WIKI_ID)
   
   local ArticlesNode = ArticlesXmlRoot.response.articles
   local ArticleMap = fogbugz_tree.createArticleMap(ArticlesNode)
   debug(ArticleMap)
   
   local ArticleId, Article
   for ArticleId, Article in pairs(ArticleMap) do
      local RawFileName = Config.destFileName(''..ArticleId..'.raw.html')
      local fetchFunction = fogcache_files.loadOrFetchArticleRawHtml
      if ''..(FetchPageId or '')==''..ArticleId or FetchPageId=='*' then
         debug('Will fetch page', ArticleId)
         fetchFunction = fogcache_files.fetchArticleRawHtml
      end
      Article.RawHtml = fetchFunction(Article, RawFileName, Config, UiSession)
      debug(Article)
   end
   
   local Suffix = Config.SUFFIX or fogcache_gen.DefaultSuffix
   
   local StartPageId = Config.START_PAGE or '1'
   
   fogbugz_tree.prepareArticleMap(ArticleMap, StartPageId)
   
   local TreeData = fogcache_tree.constructTree(ArticleMap, StartPageId, Suffix)
   
   local TreeDataJs = 'var TreeData = '..json.serialize{data=TreeData}..';'
   local TreeHash = util.md5(TreeDataJs)
   
   local SiteMapXmlSrc = fogcache_sitemap.generateSiteMap(ArticleMap, Config)
   debug(SiteMapXmlSrc)
   
   if Query.NoWrite then return end
      
   file_helper.writeFileIfChanged(Config.destFileName('tree_data.new.js'), TreeDataJs)

   for ArticleId, Article in pairs(ArticleMap) do
      local Html = fogcache_gen.fullPage(Article, TreeData, TreeHash, Config, Token)
      file_helper.writeFileIfChanged( Config.destFileName(''..Article.Id..Suffix..'.html'), Html)
   end
   
   if Config.SITE_MAP_ENABLED then
      file_helper.writeFileIfChanged(Config.destFileName('sitemap.xml'), SiteMapXmlSrc)
   end
   
   fogcache_clean.cleanFilesForDeletedPages(Config, ArticleMap)
end
